#ifndef M_HPP_INCLUDED
#define M_HPP_INCLUDED


#include <boost/statechart/event.hpp>
#include <boost/statechart/state_machine.hpp>
#include <boost/statechart/simple_state.hpp>
#include <boost/statechart/state.hpp>
#include <boost/statechart/transition.hpp>
#include <boost/statechart/custom_reaction.hpp>

namespace sc = boost::statechart;
namespace mpl = boost::mpl;

struct A1;
struct A2;
struct A3;
struct B0Dummy;
struct C0Dummy;

struct EvToB0 : sc::event<EvToB0> {};
struct EvToC0 : sc::event<EvToC0> {};

struct EvA1ToA2 : sc::event< EvA1ToA2 > {};
struct EvA2ToA3 : sc::event< EvA2ToA3 > {};
struct EvA3ToA1 : sc::event< EvA3ToA1 > {};


struct M: sc::state_machine< M, A1 >
{
    void start ();
};

struct A1 : sc::simple_state< A1, M >
{
    typedef sc::transition< EvA1ToA2, A2 > reactions;
};

struct A2 : sc::simple_state< A2, M >
{
    typedef sc::transition< EvA2ToA3, A3 > reactions;
};

struct A3 : sc::simple_state< A3, M, mpl::list< B0Dummy, C0Dummy > >
{
    typedef sc::transition< EvA3ToA1, A1 > reactions;
};

struct B0Dummy : sc::state< B0Dummy, A3::orthogonal< 0 > >
{
    typedef sc::custom_reaction< EvToB0 > reactions;

    B0Dummy(my_context ctx);
    sc::result react(const EvToB0 &);
};

struct C0Dummy : sc::state< C0Dummy, A3::orthogonal< 1 > >
{
    typedef sc::custom_reaction< EvToC0 > reactions;

    C0Dummy(my_context ctx);
    sc::result react(const EvToC0 &);
};


#endif
